@@ -17,8 +17,8 @@ package ODF::MailMerge;
 # ...but don't let CPAN or test harnes scanners see this as defining $VERSION
 { no strict 'refs'; ${__PACKAGE__."::VER"."SION"} = 999.999; }
 
-our $VERSION = '1.000'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
-our $DATE = '2023-09-12'; # DATE from Dist::Zilla::Plugin::OurDate
+our $VERSION = '1.001'; # VERSION from Dist::Zilla::Plugin::OurPkgVersion
+our $DATE = '2023-09-17'; # DATE from Dist::Zilla::Plugin::OurDate
 
 use Carp;
 our @CARP_NOT = ("ODF::lpOD_Helper", "ODF::lpOD");
@@ -27,16 +27,20 @@ use Scalar::Util qw/refaddr blessed/;
 use List::Util qw/first any none all max min sum0/;
 use List::MoreUtils qw/before after uniq/;
 use Data::Dumper::Interp 6.004
-       qw/visnew ivis dvis dvisq ivisq vis visq avis avisq addrvis/;
-use Spreadsheet::Edit::Log 1000.005 qw/oops/, ':btw=M${lno}:' ;
+       qw/visnew ivis dvis dvisq ivisq vis visq avis avisq addrvis u/;
+use Spreadsheet::Edit::Log 1000.005 'oops', ':btw=MME${lno}:' ;
 use Clone ();
 
 use ODF::lpOD;
 use ODF::lpOD_Helper 6.004 qw/:DEFAULT
                               PARA_FILTER
+                              Hr_SUBST
                               Hr_MASK
                               arraytostring hashtostring/;
 
+#use constant MM_SUBST => Hr_SUBST;
+use constant MM_SUBST => 0x42FF;
+
 use constant ROW_FILTER => "table:table-row";
 use constant CELL_FILTER => "table:table-cell";
 use constant FRAME_FILTER => "draw:frame";
@@ -51,8 +55,8 @@ use constant TABLE_SECTION_FRAME_FILTER =>
                    Hor_cond(TABLE_FILTER,SECTION_FILTER,FRAME_FILTER);
 
 use Exporter 'import';
-our @EXPORT = qw/replace_tokens/;
-our @EXPORT_OK = qw/odfmm_example_path/;
+our @EXPORT = qw/replace_tokens MM_SUBST/;
+our @EXPORT_OK = qw/odfmm_example_path $token_re/;
 our %EXPORT_TAGS = ('all' => [@EXPORT, @EXPORT_OK]);
 
 our $debug;
@@ -110,8 +114,8 @@ btw visq($_) if $debug;
     if (/^(?: nb|unfold|breakmulti
              |die|del(?:empty|row|para|=.*)
              |rep(?:_first|_notfirst|_mid|_last|=.*)
-             |rmsb  # remove shared border between replicates
-             |span  # span down through empty cells below
+             |rmsb    # remove shared border between replicates
+             |span[rd]
              |reptag=.*
           )$/xs) {
       push @std_mods, $_;
@@ -202,8 +206,8 @@ sub _fmt_tokhash($) {
 # of the same tokname in a single paragraph) the 2nd pass has to re-process
 # a token from scratch.
 #
-# Returns () if the token should not be replaced, otherwise man details
-# including a (ref to) array of [content] specs.
+# Returns () if the token should not be replaced, otherwise $content_list
+#   ($content_list is a (ref to) array of [content] specs)
 sub _get_content_list($$$$) {
   my ($m, $tokname, $users_hash, $custom_mods) = @_;
   my $val = $users_hash->{$tokname} // $users_hash->{'*'};
@@ -212,24 +216,28 @@ sub _get_content_list($$$$) {
   my $token = $m->{match};
   if (ref($val) eq "CODE") {
     my $para  = $m->{para};
-    (my $return_op, $val) = $val->($tokname, $token, $para, $custom_mods);
-    croak("callback returned Hr_SUBST without a value or vice-versa: $token")
-      if !defined($val) ^ !($return_op & Hr_SUBST);
-    return undef
+    (my $op, $val) = $val->($tokname, $token, $para, $custom_mods);
+    croak("Callback returned an invalid opcode: ",u($op))
+      if !defined($op) or $op != MM_SUBST && $op != 0;
+    croak("callback returned MM_SUBST without a value or vice-versa: $token")
+      if !defined($val) ^ !($op & MM_SUBST);
+    return ()
       unless defined($val);
   } else {
     croak "Invalid modifer ",visq(":$_")," in token $token",
           "\n(A callback is required to use custom modifiers)\n"
       for @$custom_mods;
   }
+  croak ivisq 'Value for token $token is []'
+    if ref($val) eq 'ARRAY' && @$val==0;
   my $content_list = _to_content_list($val);
-btw visnew->dvisq('CCC $content_list') if $debug;
+btw visnew->dvisq('CCC $val $content_list') if $debug;
   $content_list
 }
 
 sub _get_replicate_opts($) {
   my $std_mods = shift;
-  my ($cond_expr, $rmsb, $span);
+  my ($cond_expr, $rmsb, $spandown, $spanright);
   foreach (@$std_mods) {
     my $cexpr;
     if (/^rep/) {
@@ -258,9 +266,10 @@ sub _get_replicate_opts($) {
       }
     }
     elsif ($_ eq "rmsb") { $rmsb = 1 }
-    elsif ($_ eq "span") { $span = 1 }
+    elsif ($_ eq "spand") { $spandown = 1 }
+    elsif ($_ eq "spanr") { $spanright = 1 }
   }
- return ($cond_expr, $rmsb, $span)
+ return ($cond_expr, $rmsb, $spandown, $spanright)
 }
 
 sub _rt_dryrun($$$) {
@@ -275,7 +284,7 @@ btw dvis '_rt_dryrun $token' if $debug;
             = _get_content_list($m, $tokname, $users_hash, $custom_mods);
     next
       unless defined $content_list;
-    my ($cond_expr, $rmsb, $span) = _get_replicate_opts($std_mods);
+    my ($cond_expr, $rmsb, $spand, $spanr) = _get_replicate_opts($std_mods);
     my ($rop, $rop_name) = _para_to_rop($m->{para});
     my $tokhash_key = _mk_tokhash_key($rop, $tokname);
     if (exists $tokhash->{$tokhash_key}) {
@@ -292,7 +301,8 @@ btw dvis '_rt_dryrun $token' if $debug;
       token        => $token, # just for debugging?
       (defined($cond_expr) ? (cond_expr => $cond_expr) : ()),
       ($rmsb               ? (rmsb => 1) : ()),
-      ($span               ? (span => 1) : ()),
+      ($spand              ? (spandown => 1) : ()),
+      ($spanr              ? (spanright => 1) : ()),
     };
   }#foreach token in context
 }# _rt_dryrun
@@ -328,40 +338,77 @@ sub _do_rm_border($$$) {
 sub _do_rmtb($$) { &_do_rm_border(@_, "fo:border-top"   ) }
 sub _do_rmbb($$) { &_do_rm_border(@_, "fo:border-bottom") }
 
-sub _do_span($) {
-  my $spanning_cell = shift;
-  # This is called after substitutions.  Span the cell down over
-  # any cells below which are empty.  The empty cells might or might not
-  # be part of a replicate group!
+sub _num_empty_atorbelow($$$$) {
+  my ($table, $rx, $numrows, $cx) = @_;
+  my $num_empty = 0;
+  while ($rx + $num_empty < $numrows) {
+    my $row = $table->get_row($rx+$num_empty);
+    my $this_cell = $row->get_cell($cx);
+    if ($this_cell->Hget_text() ne "") {
+      last
+    }
+    $num_empty++;
+  }
+  $num_empty;
+}
+sub _do_span($@) {
+  my ($spanning_cell, $spandirs) = @_;
+  # This is called after substitutions.  Span the cell over
+  # cells which are now empty (paying no attention to replicate groups!)
+  my $tag = $spanning_cell->tag;
+  if ($tag eq 'table:covered-table-cell') {
+    # This cell has already been spaneed-over.  So skip it.
+    return;
+  }
+  oops unless $tag eq "table:table-cell" && $spanning_cell->first_child;
   my $table = $spanning_cell->get_parent_table;
   my ($numrows, $numcols) = $table->get_size;
   my (undef, $rx, $cx) = $spanning_cell->get_position;
+  # Don't confuse 'rspan' (rows spanned) here with ':spanr' (span-right :mod)
   my ($rspan, $cspan) = $spanning_cell->get_span;
-  # r0:before r1:before r2:$cell  r3:"" c4:"" r5:NotEmpty     (numrows==6)
-  #                     nrspan==1 =2    =3    (=4)
-  my $new_rspan = $rspan; # e.g. 1
-  while (($rx+$new_rspan) < $numrows) {
-    my $row = $table->get_row($rx+$new_rspan);
-    my $this_cell = $row->get_cell($cx);
-    last if $this_cell->Hget_text() ne "";
-    btw dvis 'DELETING CONTENT OF $this_cell $rx+$new_rspan: ',visq($this_cell->Hget_text) if $debug;
-    # Entirely delete paragraphs (and nested tables or sections...)
-    # from cells to be covered; otherwise they seem to end up in the
-    # spanning cell (contrary to what ODF::lpOD docs imply about covered cells).
-    foreach ($this_cell->children) {
-      btw "    (deleting $_)" if $debug;
-      $_->delete;
-    }
-    ++$new_rspan;
+  oops if $rspan != 1 || $cspan != 1; #not handled
+  my $new_rspan = $rspan;
+  my $new_cspan = $cspan;
+  if ($spandirs->{down} && $rx < $numrows-1) {
+    # r0:before r1:before r2:$cell  r3:"" c4:"" r5:NotEmpty     (numrows==6)
+    #                     nrspan==1 =2    =3    (=4)
+    $new_rspan = 1 + _num_empty_atorbelow($table, $rx+1, $numrows, $cx);
   }
-  if ($new_rspan != $rspan) {
-    btw dvis 'SET SPAN: $rspan, $new_rspan $rx $cx $cspan $spanning_cell' if $debug;
-    $spanning_cell->set_span(rows => $new_rspan, columns => $cspan);
+  if ($spandirs->{right} && $cx < $numcols-1) {
+    for (my $this_cx = $cx+1; $this_cx < $numcols; $this_cx++) {
+      my $vempty = _num_empty_atorbelow($table, $rx, $numrows, $this_cx);
+      # If also vertically spanning, horiz. spanning can only be done
+      # if the spanned columns are empty in all vertically-spanned rows,
+      # i.e. no non-empty cell will be covered.
+      last if $vempty < $new_rspan; # vempty >= 1 to cover 1st row etc.
+      $new_cspan++;
+    }
+  }
+  # Entirely delete paragraphs (and nested tables or sections...)
+  # from cells to be covered; otherwise the content seems to end up in the
+  # spanning cell (contrary to what ODF::lpOD docs imply about covered cells).
+  for my $this_rx ($rx .. $rx+$new_rspan-1) {
+    my $row = $table->get_row($this_rx);
+    for my $this_cx ($cx .. $cx+$new_cspan-1) {
+      next if $this_rx==$rx && $this_cx==$cx; # the surviving (spanning) cell
+      my $cell = $row->get_cell($this_cx);
+      btw dvis 'DELETING CONTENT OF $cell $this_rx $this_cx ($numcols $numrows $new_rspan $new_cspan): ',
+          visq($cell->Hget_text) if $debug;
+      foreach ($cell->children) {
+        btw "    (deleting $_)" if $debug;
+        $_->delete;
+      }
+    }
+  }
+
+  if ($new_rspan != $rspan or $new_cspan != $cspan) {
+    oops if $new_rspan <= 0 or $new_cspan <= 0;
+    $spanning_cell->set_span(rows => $new_rspan, columns => $new_cspan);
   }
 }
 
 sub _rt_dosubst($$$$$) { # returns Hreplace result list
-  my ($context, $users_hash, $tokhash, $to_deletes, $spandowns) = @_;
+  my ($context, $users_hash, $tokhash, $to_deletes, $spans_todo) = @_;
   my $doc = $context->document();
   $context->Hreplace($token_re, sub {
     my $m = shift;
@@ -385,7 +432,7 @@ btw dvis 'XX retrieved $tokhash_key -> $info $content_list' if $debug;
       # A token is not in %tokhash if it is the 2nd instance of the same
       # token in a rop (in which case multi-values are not allowed).
       $content_list
-        = _get_content_list($m, $tokname, $users_hash, $custom_mods);
+            = _get_content_list($m, $tokname, $users_hash, $custom_mods);
 btw dvis 'YY *no* info, $rop $tokname $token$token  $users_hash $content_list' if $debug;
       return(0)
         unless defined $content_list;
@@ -419,18 +466,22 @@ btw dvis 'YY *no* info, $rop $tokname $token$token  $users_hash $content_list' i
         }
       }
       elsif ($_ eq "rmsb") { } # handled in 1st pass, set {rmtb/rmbb} in $info
-      elsif ($_ eq "span") {
+      elsif (/^span[dr]$/) {
+        my $down_or_right = $_ eq 'spand' ? "down" : "right";
         # If the rop is in a replicate group (possibly by itself), then
-        # $info->{span} is set in the first replicate only.
+        # $info->{spandown} is set in the first replicate only.
         # Otherwise this is an odd case (2nd token in same paragraph)
         # where there is no $info
-        if (!$info || $info->{span}) {
-          # Do *not* look outside a Frame wrapper possibly in a cell
+        if (!$info || $info->{spandown} || $info->{spanright}) {
+          # Do *not* look outside a Frame wrapper within in a cell
           if (my $cell = $m->{para}->Hparent(CELL_FILTER, FRAME_FILTER)) {
             my $rop = $cell->parent(ROW_FILTER);
             # Record the rop so we can later check that it wasn't deleted
-            # before fiddling with the cell
-            $spandowns->{refaddr $cell} = [$rop, $cell];
+            # before fiddling with the cell.  N.B. Both :spand and :spanr
+            # may be present in the cell, not necessarily in the same {token}
+            # (although that makes no difference).
+            my $aref = $spans_todo->{refaddr $cell} //= [$rop, $cell, {}];
+            $aref->[2]->{$down_or_right} = 1;
           }
         }
       }
@@ -578,7 +629,7 @@ btw ivis 'GGG-Insert $new_rop (cloned from $templ) as PREV_SIB of $first_rop ',
             #rop          => $new_rop,
             #cond_expr    => $info->{cond_expr}, # just for debugging??
           };
-          foreach (qw/tokname token rmsb span/) {
+          foreach (qw/tokname token rmsb spandown spanright/) {
             $new_info->{$_} = $info->{$_} if exists $info->{$_};
           }
           $info = $new_info;
@@ -592,7 +643,7 @@ btw dvis 'G G-1 $i $N Created $new_key $tokhash->{$new_key}\n $info' if $debug;
             $info->{rmtb} = 1 if $i > 0;    # remove top border
           }
         }
-        delete $info->{span} if $i > 0;
+        delete $info->{spandown} if $i > 0;
       }
 btw dvis 'G G-2 $i $new_rop' if $debug;
     }# foreach $i
@@ -713,9 +764,9 @@ oops unless blessed($context);
 #      (croak if multiple values at this point)
 #
 #      Apply std_mods to the value
-#      return (Hr_SUBST, [content])
-  my (%to_deletes, %spandowns);
-  my @rr = _rt_dosubst($context, $hash, \%tokhash, \%to_deletes, \%spandowns);
+#      return (MM_SUBST, [content])
+  my (%to_deletes, %spans_todo);
+  my @rr = _rt_dosubst($context, $hash, \%tokhash, \%to_deletes, \%spans_todo);
   btw "AFTER SUBSTITUTIONS: context=", fmt_tree($context, wi => 2) if $debug;
 
 # 4. Delete rops which contained token(s) with :del* modifiers where
@@ -732,11 +783,11 @@ oops unless blessed($context);
     }
   }
 
-# 5. Span cells containing tokens with :span if cells below are empty
-  foreach (values %spandowns) {
-    my ($rop, $cell) = @$_;
+# 5. Span cells containing tokens with :spand if cells below are empty
+  foreach (values %spans_todo) {
+    my ($rop, $cell, $spandirs) = @$_;
     next if exists $to_deletes{$rop}; # was deleted above
-    _do_span($cell);
+    _do_span($cell, $spandirs);
   }
   # ??? should replace count be reduced by the number of tokens in
   # objects removed via $to_delete ???
@@ -747,10 +798,12 @@ package ODF::MailMerge::Engine;
 
 use constant TABLE_SECTION_FRAME_FILTER => ODF::MailMerge::TABLE_SECTION_FRAME_FILTER;
 use constant TABLE_FILTER => ODF::MailMerge::TABLE_FILTER;
+use constant MM_SUBST     => ODF::MailMerge::MM_SUBST;
 
 use ODF::lpOD;
 use ODF::lpOD_Helper;
 use Data::Dumper::Interp;
+use Spreadsheet::Edit::Log 1000.005 'oops', ':btw=MME${lno}:' ;
 use Carp;
 our @CARP_NOT = ("ODF::MailMerge", "ODF::lpOD_Helper", "ODF::lpOD");
 
@@ -790,6 +843,9 @@ sub new {
   },$class
 }
 
+my $__FILE__ = __FILE__;
+my $__PACKAGE__ = __PACKAGE__;
+
 sub add_record {
   my ($self, $hash, %opts) = @_;
   local $debug = $debug || $opts{debug};
@@ -824,7 +880,7 @@ sub add_record {
   # have some value.  This wrapper callback enforces that.
   my sub wrapper_cb {
     my ($tokname, $token, $para, $custom_mods) = @_;
-    my $return_op = Hr_SUBST;
+    my $return_op = MM_SUBST;
     my $key = exists($hash->{$tokname}) ? $tokname :
               exists($hash->{'*'}) ? '*' :
               croak "Unhandled token ", vis($token),
@@ -836,15 +892,26 @@ sub add_record {
       ($return_op, $val) = $val->(@_);
       croak "Unhandled token ", vis($token),
             ivis '; the callback in hash{$key} returned ($return_op,$val)\n'
-        unless ($return_op & Hr_SUBST)==0 || defined $val;
+        unless ($return_op & MM_SUBST)==0 || defined $val;
     } else {
-      croak "Unhandled token modifier ",visq(":$_") foreach @$custom_mods;
+      croak "Invalid token modifier ",visq(":$_") foreach @$custom_mods;
     }
     # FIXME: Why is _to_content_list needed here?
     return ($return_op, ODF::MailMerge::_to_content_list($val))
   }
-  ODF::MailMerge::replace_tokens($table, {'*' => \&wrapper_cb}, %opts);
-}
+  #ODF::MailMerge::replace_tokens($table, {'*' => \&wrapper_cb}, %opts);
+  my $r = eval {
+           ODF::MailMerge::replace_tokens($table, {'*' => \&wrapper_cb}, %opts)
+          };
+  if ($@) { # filter out our internal stuff to make easier to read
+    { local $_;
+      $@ =~ s/^[^\n]* called \Kat $__FILE__ line .*?(?=\s*${__PACKAGE__}::add_record)/from [MM internals]/msg
+        unless $opts{debug};
+    }
+    die "$@\n";
+  }
+  $r
+}#add_record
 
 sub finish {
   my ($self, %opts) = @_;
@@ -866,7 +933,7 @@ ODF::MailMerge - "Mail Merge" or just substitute tokens in ODF documents
 
  use ODF::lpOD;
  use ODF::lpOD_Helper;
- use ODF::MailMerge qw/replace_tokens/;
+ use ODF::MailMerge qw/replace_tokens MM_SUBST/;
 
  my $doc = odf_get_document("/path/to/file.odt");
  my $body = $doc->get_body;
@@ -901,13 +968,13 @@ ODF::MailMerge - "Mail Merge" or just substitute tokens in ODF documents
 
 =head1 DESCRIPTION
 
-This tool uses ODF::lpOD and ODF::lpOD_Helper to patch ODF documents.
+This tool uses ODF::lpOD / ODF::lpOD_Helper to patch ODF documents.
 Token strings of the form "{key}" or "{key:modifiers...}"
 are replaced with values from a hash indexed by "key".
 
 Optional :modifiers within tokens
-can change the value actually substituted or have side-effects
-such as removing lines when there is no value to substitute.
+can change the substituted value or have side-effects
+such as removing lines when the substituted value is empty.
 
 A "mail merge" function replicates a template object (e.g. table or section)
 as many times as needed to plug in values from multiple data records.
@@ -966,7 +1033,7 @@ database records, plugging in specific values from each reacord.
 Some fields may have empty ("") values in a particular record,
 in which case the containing row, paragraph etc.
 can be I<deleted to avoid leaving undesirable blank space>.
-For example a mailing list may allow for a secondary addressee line
+For example a mailing list may allow a secondary addressee line
 which is not always needed.
 
 =item 3.
@@ -993,7 +1060,7 @@ row being visited by 'apply'.
 Therefore tokens {Name} and {Address} would be
 replaced by appropriate values from the "Name" and "Address" columns.
 
-=head2 $engine = ODF::MailMerge::Engine-E<gt>new(context => $context, proto_tag => "{tag}");
+=head2 $engine = ODF::MailMerge::Engine-E<gt>new(context=>$body, proto_tag=>"{tag}");
 
 =head2 $engine = ODF::MailMerge::Engine-E<gt>new(proto_elt => $elt);
 
@@ -1002,8 +1069,9 @@ element.
 Currently only I<table> prototypes are supported, but
 but I<section>s and other ODF text wrappers may be supported later.
 
-In the first form, the string "{tag}" is searched for within C<$context>
-(e.g. the document body), and the containing Table is used as the prototype
+In the first form, the string "{tag}" is searched for
+within C<$context> (e.g. the document body),
+and the containing Table is used as the prototype
 element.  The tag string may be contained anywhere in the table, and will
 be deleted (so it has no effect on the final result).
 
@@ -1034,8 +1102,8 @@ or "{First Name:...}" .
 The hash key B<'*'> is a wildcard, used if there is no entry for
 a token name.
 
-Token names may contain internal spaces but leading and trailing spaces
-around the name (but not inside :modifiers) are ignored.
+Tokens may contain internal spaces but leading and trailing spaces
+around the token name are ignored.
 Literal : { or } characters must be backslashed i.e. \: \{  or \}.
 
 A B<hash value> may be:
@@ -1075,12 +1143,15 @@ The standard :modifiers are
 
   :breakmulti - Append newline if the value contains embedded newlines.
 
-  :span       - (only in a table cell) Span the cell down over cells below
-                which are empty. To be useful, the cell should have
+  :spand      - (only in a table cell) Span the cell down over cells
+                below which are empty. To be useful, the cell should have
                 Format->align text->Center so it can float.
 
-  :die        - Delete the containing row, frame, or paragraph if
-                all tokens with :die are empty ("") after substitution.
+  :spanr        (only in a table cell) Span the cell rightward over cells
+                which are empty.
+
+  :die        - Delete If Empty - the containing row, etc. is deleted
+                if all tokens with :die are empty ("") after substitution.
 
   :rmsb       - Remove shared borders between replicated rows
 
@@ -1169,20 +1240,20 @@ would produce
   └──────────────┴────────────────┴───────────────────────┘
 
 The B<:rep_first> modifier indicated that that template without bottom borders
-should be used for the first row in the replication set, etc.
-Conditional :rep* modifiers must be mutually exclusive.
+should be used for the first row in a multi-value replication set, etc.
+:rep* conditions must be mutually exclusive.
 
 The first template in a set may be a "regular" template row without
-conditions, as in the example above.  This is used only when none of the
+conditions, as in the example above.  it is used only when none of the
 conditional templates apply.  In the above example that was never because
 the conditional templates coverd every situation; however the "regular"
 template would be used if there was only one "replicate", i.e. all {token}s
 had only a single value.
 
-B<:rep=EXPR> is the most general form.  EXPR is a Perl expression using
+B<:rep=EXPR> is a general conditional.  EXPR is a Perl expression using
 variables C<$i> and C<$N>, and which evaluates to
 true when the template should be instantiated.  C<$i> will hold the current
-replicate index (first is zero), and C<$N> holds the total number of
+replicate index (first is zero), and C<$N> the total number of
 rows in the replication set.
 
   "Friendly" conditional     Equivalent
@@ -1192,11 +1263,9 @@ rows in the replication set.
        :rep_only               :rep= $N==1
 
 The above example is not very compelling because the C<:rmsb> modifier provides
-built-in support for removing shared borders.
-
-The following allows odd & even rows to have distinctive formatting
-(e.g. different background colors).  The first row is always "even"
-but odd & even alternatives are given for middle and last rows:
+built-in support for removing shared borders.  Here is a more interesting
+example which formats odd & even replicates distinctively
+(such as different background colors):
 
   ┌─────────────────────────────────────────────────────────────┐
   │EVEN (first)   {Token Name:rep_first}                        │
@@ -1216,7 +1285,7 @@ but odd & even alternatives are given for middle and last rows:
 
 =head2 CALLBACKS
 
-If a hash value is a sub reference, the sub is called with args
+If a hash value is a reference to a sub, the sub is called with args
 
   ($token_name, $token, $para, $custom_mods)
 
@@ -1233,10 +1302,9 @@ Note: An exception occurs if unrecognized :modifiers are encountered
 when a callback is not being used.
 
 The callback's return values indicate whether and how to replace
-the token.  The protocol uses the Hr_* constants exported
-by L<ODF::lpOD_Helper>:
+the token:
 
-  return(Hr_SUBST, <value>)
+  return(MM_SUBST, <value>)
 
 B<< <value> >> may be any of the allowed hash values (except for a callback).
 If a [list of values] is returned and there
@@ -1249,8 +1317,6 @@ The token is not replaced, but left as-is, and processing continues.
 This only makes sense if the token will somehow be processed later,
 for example via a separate call to C<replace_tokens>.
 
-=for future FIXME: Define local MM_SUBST to avoid showing Hr_* dependencies?
-
 =head1 COMPLETE EXAMPLE
 
 A complete example application is included in the distribution.
@@ -1258,8 +1324,7 @@ To display the path on your system, run
 
   perl -MODF::MailMerge=:all -C -E 'say odfmm_example_path'
 
-(something
-like C<.../site_perl/5.xx.yy/auto/share/dist/ODF-MailMerge/examples/>)
+  # .../site_perl/5.xx.yy/auto/share/dist/ODF-MailMerge/examples/
 
 =head1 SEE ALSO
 
